﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

namespace TimingLinq
{
    /// <summary>
    /// Measure the execution time of different LINQ expressions.
    /// Faster: where ... select
    /// Slower: select ... where
    /// </summary>
    static class Program
    {
        static readonly int MAXNo = 3000000;

        static void Main()
        {
            List<Elem> elements = new List<Elem>(MAXNo);
            Intialize(elements);

            IEnumerable<Elem> enumerable = from e in elements where e.Id == MAXNo / 20 select e;
            TimeIt(enumerable, "where select, query syntax");

            enumerable = elements.Where(e => e.Id == MAXNo / 20).Select(e => e);
            TimeIt(enumerable, "where select, method syntax");

            enumerable = elements.Select(e => e).Where(e => e.Id == MAXNo / 20);
            TimeIt(enumerable, "select where, method syntax");

            enumerable = elements.Where(e => e.Id == MAXNo / 20);
            TimeIt(enumerable, "where where, no explicit select, method syntax");

            elements.Where(e => e.Id == MAXNo / 20).TimeIt("dfsdf");
        }

        private static void Intialize(ICollection<Elem> elements)
        {
            Random rand = new Random();

            for (int i = 0; i < MAXNo; i++)
            {
                int id = rand.Next(MAXNo / 10);
                string name = "name " + i;
                Elem item = new Elem { Id = id, Name = name };
                elements.Add(item);
            }
        }

        private static void TimeIt(IEnumerable<Elem> enumerable, String comment)
        {
            Stopwatch stopwatch = Stopwatch.StartNew();
            Console.WriteLine("length: {0}", enumerable.Count());
            stopwatch.Stop();
            Console.WriteLine("{0}: {1} seconds", comment, stopwatch.Elapsed);
        }
    }

    static class Helper
    {
        /// <summary>
        /// Extension method
        /// </summary>
        /// <param name="enumerable"></param>
        /// <param name="comment"></param>
        public static void TimeIt(this IEnumerable<Elem> enumerable, String comment)
        {
            Stopwatch stopwatch = Stopwatch.StartNew();
            Console.WriteLine("length: {0}", enumerable.Count());
            stopwatch.Stop();
            Console.WriteLine("{0}: {1} seconds", comment, stopwatch.Elapsed);
        }
    }

    class Elem
    {
        public int Id { get; set; }

        public string Name { get; set; }

        public override string ToString()
        {
            return string.Format("Id: {0}, Name: {1}", Id, Name);
        }
    }
}
